import 'package:agora_rtc_wrapper/agora_rtc_wrapper.dart';
import 'package:flutter/material.dart';

import 'entry_stub.dart' if (dart.library.io) 'impl/entry_stub.impl.dart';

/// Created by taohid on 15,May, 2020
/// Email: taohid32@gmail.com

abstract class AgoraRtcWrapper {
  static final AgoraRtcWrapper instance = createWrapper();
  factory AgoraRtcWrapper() => instance;

  void init(String appId);
  void listen({
    OnCallDuration onCallDuration,
    OnError onError,
    OnJoinSuccess onJoinSuccess,
    VoidCallback onLeaveChannel,
    VoidCallback onConnectionLost,
    OnUserJoined onUserJoined,
    OnUserOffline onUserOffline,
    OnInfoStats onInfoStats,
    OnFirstRemoteVideoFrame onFirstRemoteVideoFrame,
  });

  List<int> get getUsers;
  bool get isCallActive;
  int get getDuration;
  Widget get getSelfVideoView;
  Widget getVideoView(int uid);

  void joinChannel(String channelName);
  void enableLog();
  void muteAudio(bool mute);
  void switchCamera();
  void endCall();
  void dispose();

  Widget getDefaultController({
    @required VoidCallback onCallEnd,
    @required ValueChanged<bool> onMute,
    Color selectedButtonColor,
  });
}

typedef OnCallDuration = void Function(
  int totalSecond,
);

typedef OnError = void Function(
  dynamic code,
);

typedef OnJoinSuccess = void Function(
  String channel,
  int uid,
  int elapsed,
);

typedef OnUserJoined = void Function(
  int uid,
  int elapsed,
);

/// There are two types of user offline reason
/// 1. Leave the channel:
/// 2. Drop offline:
typedef OnUserOffline = void Function(
  int uid,
  int reason,
);

typedef OnFirstRemoteVideoFrame = void Function(
  int uid,
  int width,
  int height,
  int elapsed,
);

typedef OnInfoStats = void Function(RtcStats stats);
